/*
 * SPDX-FileCopyrightText: 2022 The HedgeDoc developers (see AUTHORS file)
 *
 * SPDX-License-Identifier: AGPL-3.0-only
 */
import { useApplicationState } from '../../../../../hooks/common/use-application-state'
import { Logger } from '../../../../../utils/logger'
import { useUiNotifications } from '../../../../notifications/ui-notification-boundary'
import type { YDocMessageTransporter } from '@hedgedoc/commons'
import { MessageType } from '@hedgedoc/commons'
import { useRouter } from 'next/router'
import { useCallback, useEffect } from 'react'

const logger = new Logger('UseOnNoteDeleted')

/**
 * Hook that redirects the user to the history page and displays a notification when the note is deleted.
 *
 * @param websocketConnection The websocket connection that emits the deletion event
 */
export const useOnNoteDeleted = (websocketConnection: YDocMessageTransporter): void => {
  const router = useRouter()
  const noteTitle = useApplicationState((state) => state.noteDetails.title)
  const { dispatchUiNotification } = useUiNotifications()

  const noteDeletedHandler = useCallback(() => {
    dispatchUiNotification('notifications.noteDeleted.title', 'notifications.noteDeleted.text', {
      titleI18nOptions: {
        noteTitle
      }
    })
    router?.push('/history').catch((error: Error) => {
      logger.error(`Error while redirecting to /history`, error)
    })
  }, [router, noteTitle, dispatchUiNotification])

  useEffect(() => {
    websocketConnection.on(String(MessageType.DOCUMENT_DELETED), noteDeletedHandler)
    return () => {
      websocketConnection.off(String(MessageType.DOCUMENT_DELETED), noteDeletedHandler)
    }
  })
}
